@@ -37,6 +37,7 @@
 #include "config.h"
 
 #include "torrent/exceptions.h"
+#include "torrent/chunk_manager.h"
 
 #include "chunk_list.h"
 #include "chunk.h"
@@ -68,7 +69,7 @@ ChunkList::is_queued(ChunkListNode* node) {
 
 bool
 ChunkList::has_chunk(size_type index, int prot) const {
-  return Base::at(index).is_valid() && Base::at(index).chunk()->has_permissions(prot);
+  return base_type::at(index).is_valid() && base_type::at(index).chunk()->has_permissions(prot);
 }
 
 void
@@ -76,7 +77,7 @@ ChunkList::resize(size_type s) {
   if (!empty())
     throw internal_error("ChunkList::resize(...) called on an non-empty object.");
 
-  Base::resize(s);
+  base_type::resize(s);
 
   uint32_t index = 0;
 
@@ -93,9 +94,7 @@ ChunkList::clear() {
       throw internal_error("ChunkList::clear() called but a node in the queue is still referenced.");
     
     (*itr)->dec_rw();
-
-    delete (*itr)->chunk();
-    (*itr)->set_chunk(NULL);
+    clear_chunk(*itr);
   }
 
   m_queue.clear();
@@ -105,25 +104,35 @@ ChunkList::clear() {
       std::find_if(begin(), end(), std::mem_fun_ref(&ChunkListNode::writable)) != end())
     throw internal_error("ChunkList::clear() called but a valid node was found.");
 
-  Base::clear();
+  base_type::clear();
 }
 
 ChunkHandle
 ChunkList::get(size_type index, bool writable) {
-  ChunkListNode* node = &Base::at(index);
+  ChunkListNode* node = &base_type::at(index);
 
-  if (!node->is_valid() || (writable && !node->chunk()->is_writable())) {
+  if (!node->is_valid()) {
     CreateChunk chunk = m_slotCreateChunk(index, writable);
 
-    if (chunk.first == NULL) {
-      ChunkHandle c;
-      c.set_error_number(chunk.second);
-      
-      return c;
+    if (chunk.first == NULL)
+      return ChunkHandle::from_error(chunk.second);
+
+    // Would be cleaner to do this before creating the chunk.
+    if (!m_manager->allocate(chunk.first->chunk_size())) {
+      delete chunk.first;
+      return ChunkHandle::from_error(rak::error_number::e_nomem);
     }
 
-    if (node->is_valid())
-      delete node->chunk();
+    node->set_chunk(chunk.first);
+    node->set_time_modified(rak::timer());
+
+  } else if (writable && !node->chunk()->is_writable()) {
+    CreateChunk chunk = m_slotCreateChunk(index, writable);
+
+    if (chunk.first == NULL)
+      return ChunkHandle::from_error(chunk.second);
+
+    delete node->chunk();
 
     node->set_chunk(chunk.first);
     node->set_time_modified(rak::timer());
@@ -174,124 +183,200 @@ ChunkList::release(ChunkHandle* handle) {
     }
 
   } else {
-    if (handle->object()->references() == 1) {
+    handle->object()->dec_references();
+
+    if (handle->object()->references() == 0) {
       if (is_queued(handle->object()))
         throw internal_error("ChunkList::release(...) tried to unmap a queued chunk.");
 
-      delete handle->object()->chunk();
-      handle->object()->set_chunk(NULL);
+      clear_chunk(handle->object());
     }
-
-    handle->object()->dec_references();
   }
 
   handle->clear();
 }
 
+inline void
+ChunkList::clear_chunk(ChunkListNode* node) {
+  if (!node->is_valid())
+    throw internal_error("ChunkList::clear_chunk(...) !node->is_valid().");
+
+  uint32_t size = node->chunk()->chunk_size();
+
+  delete node->chunk();
+  node->set_chunk(NULL);
+
+  m_manager->deallocate(size);
+}
+
 inline bool
-ChunkList::sync_chunk(ChunkListNode* node, int flags, bool cleanup) {
+ChunkList::sync_chunk(ChunkListNode* node, std::pair<int,bool> options) {
   if (node->references() <= 0 || node->writable() <= 0)
     throw internal_error("ChunkList::sync_chunk(...) got a node with invalid reference count.");
 
-  // Using sync for the time being.
-  if (!node->chunk()->sync(flags))
+  if (!node->chunk()->sync(options.first))
     return false;
 
   node->set_sync_triggered(true);
 
-  if (!cleanup)
+  if (!options.second)
     return true;
 
   node->dec_rw();
  
-  if (node->references() == 0) {
-    delete node->chunk();
-    node->set_chunk(NULL);
-  }
+  if (node->references() == 0)
+    clear_chunk(node);
 
   return true;
 }
 
-unsigned int
-ChunkList::sync_all(int flags) {
-  std::sort(m_queue.begin(), m_queue.end());
+uint32_t
+ChunkList::sync_chunks(int flags) {
+  Queue::iterator split;
 
-  unsigned int failed = 0;
-  Queue::iterator split = m_queue.begin();
-
-  for (Queue::iterator itr = split, last = m_queue.end(); itr != last; ++itr)
-    if (!sync_chunk(*itr, flags, true)) {
-      // Makes sure the failed chunks don't get erased.
-      std::iter_swap(itr, split++);
-      failed++;
-    }
-
-  m_queue.erase(split, m_queue.end());
-  return failed;
-}
-
-unsigned int
-ChunkList::sync_periodic() {
-  if (std::find_if(m_queue.begin(), m_queue.end(), rak::equal(0, std::mem_fun(&ChunkListNode::writable)))
-      != m_queue.end())
-    throw internal_error("ChunkList::sync_periodic() found a chunk with writable == 0.");
-
-  // Chunks that still have writers are moved infront of split. Those
-  // chunks were once released by the last writer, and later grabbed
-  // again. This should be relatively rare and should not cause any
-  // problems.
-  //
-  // Using std::stable_partition to keep the index order relatively
-  // intact for quicker sorting.
-  Queue::iterator split = std::stable_partition(m_queue.begin(), m_queue.end(), rak::not_equal(1, std::mem_fun(&ChunkListNode::writable)));
-
-  // Do various partitioning before sorting by index, so that when we
-  // call sync we always increase the index of the chunk.
-  //
-  // Some times when we sync there are users holding write references
-  // to chunks in the queue. These won't explicitly be synced, but the
-  // kernel might do so anyway if it lies in its path, so we don't
-  // sync those chunks.
-
-  bool force;
-
-  if (std::distance(split, m_queue.end()) > (difference_type)m_maxQueueSize)
-    force = true;
-  else if (std::for_each(split, m_queue.end(), chunk_list_earliest_modified()).m_time + rak::timer::from_seconds(m_timeoutSync) >= cachedTime)
-    force = false;
+  if (flags & sync_all)
+    split = m_queue.begin();
   else
-    return 0;
+    split = std::stable_partition(m_queue.begin(), m_queue.end(), rak::not_equal(1, std::mem_fun(&ChunkListNode::writable)));
+
+  // Allow a flag that does more culling, so that we only get large
+  // continous sections.
+  //
+  // How does this interact with timers, should be make it so that
+  // only areas with timers are (preferably) synced?
 
-  // The queue contains pointer of objects in a vector, so we can sort
-  // by the pointer value.
   std::sort(split, m_queue.end());
+  
+  // If we got enough diskspace and have not requested safe syncing,
+  // then sync all chunks with MS_ASYNC.
+  if (!(flags & (sync_safe | sync_sloppy)))
+    if (m_manager->safe_sync() || m_slotFreeDiskspace() <= m_manager->safe_free_diskspace())
+      flags |= sync_safe;
+    else
+      flags |= sync_force;
 
-  unsigned int failed = 0;
+  if ((flags & sync_use_timeout) && !(flags & sync_force))
+    split = partition_optimize(split, m_queue.end(), 50, 5, false);
+
+  uint32_t failed = 0;
 
   for (Queue::iterator itr = split, last = m_queue.end(); itr != last; ++itr) {
-    // Do first async on the chunk, then sync at the next periodic
-    // call.
-    if (!(*itr)->sync_triggered()) {
-      if (!sync_chunk(*itr, MemoryChunk::sync_async, false))
-        failed++;
+    
+    // We can easily skip pieces by swap_iter, so there should be no
+    // problem being selective about the ranges we sync.
 
+    // Use a function for checking the next few chunks and see how far
+    // we want to sync. When we want to sync everything use end. Call
+    // before the loop, or add a check.
+
+    // if we don't want to sync, swap and break.
+
+    std::pair<int,bool> options = sync_options(*itr, flags);
+
+    if (!sync_chunk(*itr, options)) {
       std::iter_swap(itr, split++);
-
-    } else if (force || (*itr)->time_modified() + rak::timer::from_seconds(m_timeoutSafeSync) >= cachedTime) {
-      // When constrained of space, it triggers sync just one period
-      // after async. Else it waits for the safe sync timeout.
-      if (!sync_chunk(*itr, MemoryChunk::sync_sync, true)) {
-        failed++;
-        std::iter_swap(itr, split++);
-      }
-
-    } else {
-      std::iter_swap(itr, split++);
+      
+      failed++;
+      continue;
     }
+
+    (*itr)->set_sync_triggered(true);
+
+    if (!options.second)
+      std::iter_swap(itr, split++);
   }
 
   m_queue.erase(split, m_queue.end());
+
+  // The caller must either make sure that it is safe to close the
+  // download or set the sync_ignore_error flag.
+  if (failed && !(flags & sync_ignore_error))
+    m_slotStorageError("Could not sync chunk: " + std::string(rak::error_number::current().c_str()));
+
   return failed;
 }
 
+std::pair<int, bool>
+ChunkList::sync_options(ChunkListNode* node, int flags) {
+  // Using if statements since some linkers have problem with static
+  // const int members inside the ?: operators. The compiler should
+  // be optimizing this anyway.
+
+  if (flags & sync_force) {
+
+    if (flags & sync_safe)
+      return std::make_pair(MemoryChunk::sync_sync, true);
+    else
+      return std::make_pair(MemoryChunk::sync_async, true);
+
+  } else if (flags & sync_safe) {
+      
+    if (node->sync_triggered())
+      return std::make_pair(MemoryChunk::sync_sync, true);
+    else
+      return std::make_pair(MemoryChunk::sync_async, false);
+
+  } else {
+    return std::make_pair(MemoryChunk::sync_async, true);
+  }
+}
+
+// Using a rather simple algorithm for now. This should really be more
+// robust against holes withing otherwise compact ranges and take into
+// consideration chunk size.
+inline ChunkList::Queue::iterator
+ChunkList::seek_range(Queue::iterator first, Queue::iterator last) {
+  uint32_t prevIndex = (*first)->index();
+
+  while (++first != last) {
+    if ((*first)->index() - prevIndex > 5)
+      break;
+
+    prevIndex = (*first)->index();
+  }
+
+  return first;
+}
+
+inline bool
+ChunkList::check_node(ChunkListNode* node) {
+  return
+    node->time_modified() != rak::timer() &&
+    node->time_modified() + rak::timer::from_seconds(m_manager->timeout_sync()) < cachedTime;
+}
+
+// Optimize the selection of chunks to sync. Continuous regions are
+// preferred, while if too fragmented or if too few chunks are
+// available it skips syncing of all chunks.
+
+ChunkList::Queue::iterator
+ChunkList::partition_optimize(Queue::iterator first, Queue::iterator last, int weight, int maxDistance, bool dontSkip) {
+  for (Queue::iterator itr = first; itr != last;) {
+    Queue::iterator range = seek_range(itr, last);
+
+    bool required = std::find_if(itr, range, std::bind1st(std::mem_fun(&ChunkList::check_node), this)) != range;
+    dontSkip = dontSkip || required;
+
+    if (!required && std::distance(itr, range) < maxDistance) {
+      // Don't sync this range.
+      unsigned int l = std::min(range - itr, itr - first);
+      std::swap_ranges(first, first + l, range - l);
+
+      first += l;
+
+    } else {
+      // This probably increases too fast.
+      weight -= std::distance(itr, range) * std::distance(itr, range);
+    }
+
+    itr = range;
+  }
+
+  // These values are all arbritrary...
+  if (!dontSkip && weight > 0)
+    return last;
+
+  return first;
+}
+
 }
