@@ -40,15 +40,16 @@
 #include <limits>
 
 #include "data/chunk_list.h"
-#include "download/download_info.h"
 #include "protocol/handshake_manager.h"
 #include "protocol/peer_connection_base.h"
 #include "tracker/tracker_manager.h"
 #include "torrent/exceptions.h"
 
+#include "available_list.h"
 #include "choke_manager.h"
 #include "chunk_selector.h"
 #include "chunk_statistics.h"
+#include "download_info.h"
 #include "download_main.h"
 
 namespace torrent {
@@ -61,14 +62,10 @@ DownloadMain::DownloadMain() :
   m_chunkSelector(new ChunkSelector),
   m_chunkStatistics(new ChunkStatistics),
 
-  m_started(false),
-  m_isOpen(false),
-  m_endgame(false),
-
   m_uploadThrottle(NULL),
   m_downloadThrottle(NULL) {
 
-  m_connectionList = new ConnectionList(m_info);
+  m_connectionList = new ConnectionList(this);
   m_chokeManager = new ChokeManager(m_connectionList);
 
   m_delegator.slot_chunk_find(rak::make_mem_fun(m_chunkSelector, &ChunkSelector::find));
@@ -77,10 +74,12 @@ DownloadMain::DownloadMain() :
   m_delegator.transfer_list()->slot_canceled(std::bind1st(std::mem_fun(&ChunkSelector::not_using_index), m_chunkSelector));
   m_delegator.transfer_list()->slot_queued(std::bind1st(std::mem_fun(&ChunkSelector::using_index), m_chunkSelector));
   m_delegator.transfer_list()->slot_completed(std::bind1st(std::mem_fun(&DownloadMain::receive_chunk_done), this));
+  m_delegator.transfer_list()->slot_corrupt(std::bind1st(std::mem_fun(&DownloadMain::receive_corrupt_chunk), this));
 
   m_taskTrackerRequest.set_slot(rak::mem_fn(this, &DownloadMain::receive_tracker_request));
 
   m_chunkList->slot_create_chunk(rak::make_mem_fun(&m_content, &Content::create_chunk));
+  m_chunkList->slot_free_diskspace(rak::make_mem_fun(m_content.entry_list(), &EntryList::free_diskspace));
 }
 
 DownloadMain::~DownloadMain() {
@@ -99,7 +98,7 @@ DownloadMain::~DownloadMain() {
 
 void
 DownloadMain::open() {
-  if (is_open())
+  if (info()->is_open())
     throw internal_error("Tried to open a download that is already open");
 
   m_content.entry_list()->open();
@@ -107,18 +106,18 @@ DownloadMain::open() {
   m_chunkList->resize(m_content.chunk_total());
   m_chunkStatistics->initialize(m_content.chunk_total());
 
-  m_isOpen = true;
+  info()->set_open(true);
 }
 
 void
 DownloadMain::close() {
-  if (is_active())
+  if (info()->is_active())
     throw internal_error("Tried to close an active download");
 
-  if (!is_open())
+  if (!info()->is_open())
     return;
 
-  m_isOpen = false;
+  info()->set_open(false);
 
   m_trackerManager->close();
   m_delegator.transfer_list()->clear();
@@ -131,56 +130,36 @@ DownloadMain::close() {
   m_chunkStatistics->clear();
   m_chunkList->clear();
   m_chunkSelector->cleanup();
-
-  m_peerList.clear();
 }
 
 void DownloadMain::start() {
-  if (!is_open())
+  if (!info()->is_open())
     throw client_error("Tried to start a closed download");
 
-  if (is_active())
+  if (info()->is_active())
     throw client_error("Tried to start an active download");
 
-  m_started = true;
+  info()->set_active(true);
   m_lastConnectedSize = 0;
 
-  m_endgame = false;
   m_delegator.set_aggressive(false);
   update_endgame();  
 
   receive_connect_peers();
 }  
 
-struct peer_connection_socket_address : public std::unary_function<const PeerConnectionBase*, const rak::socket_address&> {
-  const rak::socket_address& operator () (const PeerConnectionBase* p) const {
-    return *p->peer_info()->socket_address();
-  }
-};
-
 void
 DownloadMain::stop() {
-  if (!m_started)
+  if (!info()->is_active())
     return;
 
   // Set this early so functions like receive_connect_peers() knows
   // not to eat available peers.
-  m_started = false;
+  info()->set_active(false);
 
   m_slotStopHandshakes(this);
 
-  // Save the addresses we are connected to, so we don't need to
-  // perform alot of requests to the tracker when restarting the
-  // torrent. Consider saving these in the torrent file when dumping
-  // it.
-  std::list<rak::socket_address> addressList;
-  std::transform(connection_list()->begin(), connection_list()->end(), std::back_inserter(addressList), peer_connection_socket_address());
-
-  addressList.sort();
-  available_list()->insert(&addressList);
-
-  while (!connection_list()->empty())
-    connection_list()->erase(connection_list()->front());
+  connection_list()->erase_remaining(connection_list()->begin(), ConnectionList::disconnect_available);
 
   priority_queue_erase(&taskScheduler, &m_taskTrackerRequest);
 }
@@ -200,10 +179,8 @@ DownloadMain::get_bytes_left() const {
 
 void
 DownloadMain::update_endgame() {
-  if (!m_endgame && m_content.chunks_completed() + m_delegator.transfer_list()->size() + 5 >= m_content.chunk_total()) {
-    m_endgame = true;
+  if (!m_delegator.get_aggressive() && m_content.chunks_completed() + m_delegator.transfer_list()->size() + 5 >= m_content.chunk_total())
     m_delegator.set_aggressive(true);
-  }
 }
 
 void
@@ -216,15 +193,35 @@ DownloadMain::receive_chunk_done(unsigned int index) {
   m_slotHashCheckAdd(handle);
 }
 
+void
+DownloadMain::receive_corrupt_chunk(PeerInfo* peerInfo) {
+  peerInfo->set_failed_counter(peerInfo->failed_counter() + 1);
+
+  // Just use some very primitive heuristics here to decide if we're
+  // going to disconnect the peer. Also, consider adding a flag so we
+  // don't recalculate these things whenever the peer reconnects.
+
+  // That is... non at all ;)
+  connection_list()->erase(peerInfo, ConnectionList::disconnect_unwanted);
+}
+
 void
 DownloadMain::receive_connect_peers() {
-  if (!m_started)
+  if (!info()->is_active())
     return;
 
-  while (!available_list()->empty() &&
+  AvailableList::AddressList* alist = peer_list()->available_list()->buffer();
+
+  if (!alist->empty()) {
+    alist->sort();
+    peer_list()->available_list()->insert(alist);
+    alist->clear();
+  }
+
+  while (!peer_list()->available_list()->empty() &&
          connection_list()->size() < connection_list()->get_min_size() &&
          connection_list()->size() + m_slotCountHandshakes(this) < connection_list()->get_max_size()) {
-    rak::socket_address sa = available_list()->pop_random();
+    rak::socket_address sa = peer_list()->available_list()->pop_random();
 
     if (connection_list()->find(sa) == connection_list()->end())
       m_slotStartHandshake(sa, this);
@@ -233,7 +230,7 @@ DownloadMain::receive_connect_peers() {
 
 void
 DownloadMain::receive_tracker_success() {
-  if (!m_started)
+  if (!info()->is_active())
     return;
 
   priority_queue_erase(&taskScheduler, &m_taskTrackerRequest);
