@@ -40,17 +40,14 @@
 #include <rak/functional.h>
 #include <sigc++/signal.h>
 
-#include "available_list.h"
+#include "globals.h"
+
 #include "connection_list.h"
-#include "peer_list.h"
 #include "delegator.h"
 
 #include "data/content.h"
 #include "data/chunk_handle.h"
-#include "protocol/peer_info.h"
-#include "globals.h"
-
-#include "download/download_info.h"
+#include "torrent/peer_list.h"
 
 namespace torrent {
 
@@ -76,9 +73,6 @@ public:
   void                start();
   void                stop();
 
-  bool                is_open() const                            { return m_isOpen; }
-  bool                is_active() const                          { return m_started; }
-
   ChokeManager*       choke_manager()                            { return m_chokeManager; }
   TrackerManager*     tracker_manager()                          { return m_trackerManager; }
   TrackerManager*     tracker_manager() const                    { return m_trackerManager; }
@@ -93,7 +87,6 @@ public:
   Content*            content()                                  { return &m_content; }
   Delegator*          delegator()                                { return &m_delegator; }
 
-  AvailableList*      available_list()                           { return &m_availableList; }
   ConnectionList*     connection_list()                          { return m_connectionList; }
   PeerList*           peer_list()                                { return &m_peerList; }
 
@@ -103,7 +96,6 @@ public:
   ThrottleList*       download_throttle()                        { return m_downloadThrottle; }
   void                set_download_throttle(ThrottleList* t)     { m_downloadThrottle = t; }
 
-  bool                get_endgame() const                        { return m_endgame; }
   uint64_t            get_bytes_left() const;
 
   // Carefull with these.
@@ -123,6 +115,7 @@ public:
 
   void                receive_connect_peers();
   void                receive_chunk_done(unsigned int index);
+  void                receive_corrupt_chunk(PeerInfo* peerInfo);
 
   void                receive_tracker_success();
   void                receive_tracker_request();
@@ -149,13 +142,9 @@ private:
   Content             m_content;
   Delegator           m_delegator;
 
-  AvailableList       m_availableList;
   ConnectionList*     m_connectionList;
   PeerList            m_peerList;
 
-  bool                m_started;
-  bool                m_isOpen;
-  bool                m_endgame;
   uint32_t            m_lastConnectedSize;
 
   ThrottleList*       m_uploadThrottle;
