@@ -53,6 +53,7 @@
 #include "torrent/object.h"
 #include "tracker/tracker_manager.h"
 
+#include "available_list.h"
 #include "chunk_selector.h"
 
 #include "download_wrapper.h"
@@ -69,13 +70,15 @@ DownloadWrapper::DownloadWrapper() :
   m_main.tracker_manager()->set_info(info());
   m_main.tracker_manager()->slot_success(rak::make_mem_fun(this, &DownloadWrapper::receive_tracker_success));
   m_main.tracker_manager()->slot_failed(rak::make_mem_fun(this, &DownloadWrapper::receive_tracker_failed));
+
+  m_main.chunk_list()->slot_storage_error(rak::make_mem_fun(this, &DownloadWrapper::receive_storage_error));
 }
 
 DownloadWrapper::~DownloadWrapper() {
-  if (m_main.is_active())
+  if (info()->is_active())
     m_main.stop();
 
-  if (m_main.is_open())
+  if (info()->is_open())
     close();
 
   delete m_hash;
@@ -91,8 +94,6 @@ DownloadWrapper::initialize(const std::string& hash, const std::string& id) {
 
   info()->slot_stat_left() = rak::make_mem_fun(&m_main, &DownloadMain::get_bytes_left);
 
-  m_main.chunk_list()->set_max_queue_size((32 << 20) / m_main.content()->chunk_size());
-
   m_main.connection_list()->slot_connected(rak::make_mem_fun(this, &DownloadWrapper::receive_peer_connected));
   m_main.connection_list()->slot_disconnected(rak::make_mem_fun(this, &DownloadWrapper::receive_peer_disconnected));
 
@@ -108,7 +109,7 @@ DownloadWrapper::initialize(const std::string& hash, const std::string& id) {
 
 void
 DownloadWrapper::open() {
-  if (m_main.is_open())
+  if (info()->is_open())
     return;
 
   m_main.open();
@@ -128,7 +129,7 @@ DownloadWrapper::close() {
   // This could/should be async as we do not care that much if it
   // succeeds or not, any chunks not included in that last
   // hash_resume_save get ignored anyway.
-  m_main.chunk_list()->sync_all(MemoryChunk::sync_async);
+  m_main.chunk_list()->sync_chunks(ChunkList::sync_all | ChunkList::sync_force | ChunkList::sync_sloppy | ChunkList::sync_ignore_error);
 
   m_main.close();
 
@@ -141,56 +142,35 @@ DownloadWrapper::is_stopped() const {
   return !m_main.tracker_manager()->is_active() && !m_main.tracker_manager()->is_busy();
 }
 
-void
-DownloadWrapper::insert_available_list(const std::string& src) {
-  std::list<rak::socket_address> l;
-
-  std::copy(reinterpret_cast<const SocketAddressCompact*>(src.c_str()),
-            reinterpret_cast<const SocketAddressCompact*>(src.c_str() + src.size() - src.size() % sizeof(SocketAddressCompact)),
-            std::back_inserter(l));
-  l.sort();
-
-  m_main.available_list()->insert(&l);
-}
-
-void
-DownloadWrapper::extract_available_list(std::string& dest) {
-  dest.reserve(m_main.available_list()->size() * sizeof(SocketAddressCompact));
-  
-  for (AvailableList::const_iterator itr = m_main.available_list()->begin(), last = m_main.available_list()->end(); itr != last; ++itr) {
-    if (itr->family() == rak::socket_address::af_inet) {
-      SocketAddressCompact sac(itr->sa_inet()->address_n(), itr->sa_inet()->port_n());
-
-      dest.append(sac.c_str(), sizeof(SocketAddressCompact));
-    }
-  }  
-}
-
-void
-DownloadWrapper::receive_keepalive() {
-  for (ConnectionList::iterator itr = m_main.connection_list()->begin(); itr != m_main.connection_list()->end(); )
-    if (!(*itr)->receive_keepalive())
-      itr = m_main.connection_list()->erase(itr);
-    else
-      itr++;
-}
-
 void
 DownloadWrapper::receive_initial_hash() {
-  if (m_main.is_active())
+  if (info()->is_active())
     throw internal_error("DownloadWrapper::receive_initial_hash() but we're in a bad state.");
 
   if (!m_hash->is_checking()) {
+    if (rak::error_number(m_hash->error_number()).is_valid())
+      info()->signal_storage_error().emit(("Hash checker was unable to map chunk: " + std::string(rak::error_number(m_hash->error_number()).c_str())));
+
     m_hash->clear();
+    m_hash->ranges().clear();
 
     // Clear after m_hash to ensure that the empty hash done signal does
     // not get passed to HashTorrent.
     m_hash->get_queue()->remove(this);
-    m_main.content()->bitfield()->unset_all();
+
+    m_main.content()->bitfield()->unallocate();
 
   } else if (m_main.content()->entry_list()->resize_all()) {
     m_hash->confirm_checked();
 
+    if (m_hash->get_queue()->has(this))
+      throw internal_error("DownloadWrapper::receive_initial_hash() found a chunk in the HashQueue.");
+
+    // Initialize the ChunkSelector here so that no chunks will be
+    // marked by HashTorrent that are not accounted for.
+    m_main.chunk_selector()->initialize(m_main.content()->bitfield(), m_main.chunk_statistics());
+    receive_update_priorities();
+
   } else {
     // We couldn't resize the files, tell the client.
     receive_storage_error("Could not resize files in the torrent.");
@@ -198,14 +178,6 @@ DownloadWrapper::receive_initial_hash() {
     // Do we clear the hash?
   }
 
-  if (m_hash->get_queue()->has(this))
-    throw internal_error("DownloadWrapper::receive_initial_hash() found a chunk in the HashQueue.");
-
-  // Initialize the ChunkSelector here so that no chunks will be
-  // marked by HashTorrent that are not accounted for.
-  m_main.chunk_selector()->initialize(m_main.content()->bitfield(), m_main.chunk_statistics());
-  receive_update_priorities();
-
   m_signalInitialHash.emit();
 }    
 
@@ -220,7 +192,7 @@ DownloadWrapper::receive_hash_done(ChunkHandle handle, const char* hash) {
   if (!handle.is_valid())
     throw internal_error("DownloadWrapper::receive_hash_done(...) called on an invalid chunk.");
 
-  if (!is_open())
+  if (!info()->is_open())
     throw internal_error("DownloadWrapper::receive_hash_done(...) called but the download is not open.");
 
   if (hash == NULL) {
@@ -266,13 +238,23 @@ DownloadWrapper::receive_hash_done(ChunkHandle handle, const char* hash) {
 
 void
 DownloadWrapper::receive_storage_error(const std::string& str) {
+//   m_main.chunk_list()->slot_storage_error(ChunkList::SlotStorageError(NULL, NULL));
+
+  m_main.stop();
+  m_main.close();
+
+//   m_main.chunk_list()->slot_storage_error(rak::make_mem_fun(this, &DownloadWrapper::receive_storage_error));
+
+  m_main.tracker_manager()->set_active(false);
+  m_main.tracker_manager()->close();
+
   info()->signal_storage_error().emit(str);
 }
 
 void
 DownloadWrapper::receive_tracker_success(AddressList* l) {
   m_main.connection_list()->set_difference(l);
-  m_main.available_list()->insert(l);
+  m_main.peer_list()->available_list()->insert(l);
   m_main.receive_connect_peers();
   m_main.receive_tracker_success();
 
@@ -291,27 +273,31 @@ DownloadWrapper::receive_peer_connected(PeerConnectionBase* peer) {
 
 void
 DownloadWrapper::receive_peer_disconnected(PeerConnectionBase* peer) {
-  m_main.receive_connect_peers();
-
   m_signalPeerDisconnected.emit(Peer(peer));
 }
 
 void
-DownloadWrapper::receive_tick() {
-  if (!m_main.is_open())
+DownloadWrapper::receive_tick(uint32_t ticks) {
+  // Trigger culling of PeerInfo's every hour. This should be called
+  // before the is_open check to ensure that stopped torrents reduce
+  // their memory usage.
+  if (ticks % 120 == 0)
+//   if (ticks % 1 == 0)
+    m_main.peer_list()->cull_peers(PeerList::cull_old || PeerList::cull_keep_interesting);
+
+  if (!info()->is_open())
     return;
 
-  unsigned int syncFailed = m_main.chunk_list()->sync_periodic();
-
-  if (m_main.is_active() && syncFailed != 0) {
-    // Need to move this stuff into a seperate function.
-    m_main.stop();
-
-    m_connectionChunkPassed.disconnect();
-    m_connectionChunkFailed.disconnect();
-
-    info()->signal_storage_error().emit("Could not sync data to disk, possibly full.");
+  // Every 2 minutes.
+  if (ticks % 4 == 0) {
+    for (ConnectionList::iterator itr = m_main.connection_list()->begin(); itr != m_main.connection_list()->end(); )
+      if (!(*itr)->receive_keepalive())
+        itr = m_main.connection_list()->erase(itr, ConnectionList::disconnect_available);
+      else
+        itr++;
   }
+
+  m_main.receive_connect_peers();
 }
 
 void
