@@ -43,6 +43,7 @@
 #include "data/chunk_list.h"
 #include "data/hash_queue.h"
 #include "data/hash_torrent.h"
+#include "download/available_list.h"
 #include "download/choke_manager.h"
 #include "download/chunk_selector.h"
 #include "download/chunk_statistics.h"
@@ -58,6 +59,7 @@
 #include "download.h"
 #include "file_list.h"
 #include "object.h"
+#include "peer_info.h"
 #include "tracker_list.h"
 
 namespace torrent {
@@ -69,7 +71,7 @@ Download::open() {
 
 void
 Download::close() {
-  if (m_ptr->main()->is_active())
+  if (m_ptr->info()->is_active())
     stop();
 
   m_ptr->close();
@@ -80,10 +82,10 @@ Download::start() {
   if (!m_ptr->hash_checker()->is_checked())
     throw client_error("Tried to start an unchecked download");
 
-  if (!m_ptr->main()->is_open())
+  if (!m_ptr->info()->is_open())
     throw client_error("Tried to start a closed download");
 
-  if (m_ptr->main()->is_active())
+  if (m_ptr->info()->is_active())
     return;
 
   m_ptr->main()->start();
@@ -107,7 +109,7 @@ Download::start() {
 
 void
 Download::stop() {
-  if (!m_ptr->main()->is_active())
+  if (!m_ptr->info()->is_active())
     return;
 
   m_ptr->main()->stop();
@@ -120,7 +122,7 @@ Download::hash_check(bool tryQuick) {
   if (m_ptr->hash_checker()->is_checking())
     return m_ptr->hash_checker()->start(tryQuick);
 
-  if (!m_ptr->main()->is_open() || m_ptr->main()->is_active())
+  if (!m_ptr->info()->is_open() || m_ptr->info()->is_active())
     throw client_error("Download::hash_check(...) called on a closed or active download.");
 
   if (m_ptr->hash_checker()->is_checked())
@@ -161,12 +163,12 @@ Download::hash_stop() {
 
 bool
 Download::is_open() const {
-  return m_ptr->main()->is_open();
+  return m_ptr->info()->is_open();
 }
 
 bool
 Download::is_active() const {
-  return m_ptr->main()->is_active();
+  return m_ptr->info()->is_active();
 }
 
 bool
@@ -231,6 +233,15 @@ Download::tracker_list() const {
   return TrackerList(m_ptr->main()->tracker_manager());
 }
 
+PeerList*
+Download::peer_list() {
+  return m_ptr->main()->peer_list();
+}
+
+const PeerList*
+Download::peer_list() const {
+  return m_ptr->main()->peer_list();
+}
 const TransferList*
 Download::transfer_list() const {
   return m_ptr->main()->delegator()->transfer_list();
@@ -275,6 +286,11 @@ Download::bytes_total() const {
   return m_ptr->main()->content()->entry_list()->bytes_size();
 }
 
+uint64_t
+Download::free_diskspace() const {
+  return m_ptr->main()->content()->entry_list()->free_diskspace();
+}
+
 uint32_t
 Download::chunks_size() const {
   return m_ptr->main()->content()->chunk_size();
@@ -302,12 +318,29 @@ Download::chunks_seen() const {
 
 void
 Download::set_chunks_done(uint32_t chunks) {
-  if (m_ptr->main()->is_open())
+  if (m_ptr->info()->is_open())
     throw input_error("Download::set_chunks_done(...) Download is open.");
 
   m_ptr->main()->content()->bitfield()->set_size_set(chunks);
 }
 
+void
+Download::set_bitfield(bool allSet) {
+  if (m_ptr->hash_checker()->is_checked() || m_ptr->hash_checker()->is_checking())
+    throw input_error("Download::set_bitfield(...) Download in invalid state.");
+
+  Bitfield* bitfield = m_ptr->main()->content()->bitfield();
+
+  bitfield->allocate();
+
+  if (allSet)
+    bitfield->set_all();
+  else
+    bitfield->unset_all();
+  
+  m_ptr->hash_checker()->ranges().clear();
+}
+
 void
 Download::set_bitfield(uint8_t* first, uint8_t* last) {
   if (m_ptr->hash_checker()->is_checked() || m_ptr->hash_checker()->is_checking())
@@ -316,9 +349,11 @@ Download::set_bitfield(uint8_t* first, uint8_t* last) {
   if (std::distance(first, last) != (ptrdiff_t)m_ptr->main()->content()->bitfield()->size_bytes())
     throw input_error("Download::set_bitfield(...) Invalid length.");
 
-  m_ptr->main()->content()->bitfield()->allocate();
-  std::memcpy(m_ptr->main()->content()->bitfield()->begin(), first, m_ptr->main()->content()->bitfield()->size_bytes());
-  m_ptr->main()->content()->bitfield()->update();
+  Bitfield* bitfield = m_ptr->main()->content()->bitfield();
+
+  bitfield->allocate();
+  std::memcpy(bitfield->begin(), first, bitfield->size_bytes());
+  bitfield->update();
   
   m_ptr->hash_checker()->ranges().clear();
 }
@@ -339,47 +374,7 @@ Download::bitfield() const {
 
 void
 Download::sync_chunks() {
-  m_ptr->main()->chunk_list()->sync_all(MemoryChunk::sync_sync);
-}
-
-void
-Download::insert_addresses(const std::string& addresses) {
-  m_ptr->insert_available_list(addresses);
-}
-
-void
-Download::extract_addresses(std::string& addresses) {
-  m_ptr->extract_available_list(addresses);
-}
-
-uint32_t
-Download::timeout_sync() const {
-  return m_ptr->main()->chunk_list()->timeout_sync();
-}
-
-void
-Download::set_timeout_sync(uint32_t seconds) {
-  m_ptr->main()->chunk_list()->set_timeout_sync(seconds);
-}
-
-uint32_t
-Download::timeout_safe_sync() const {
-  return m_ptr->main()->chunk_list()->timeout_safe_sync();
-}
-
-void
-Download::set_timeout_safe_sync(uint32_t seconds) {
-  m_ptr->main()->chunk_list()->set_timeout_safe_sync(seconds);
-}
-
-uint32_t
-Download::max_chunks_queued() const {
-  return m_ptr->main()->chunk_list()->max_queue_size();
-}
-
-void
-Download::set_max_chunks_queued(uint32_t chunks) {
-  m_ptr->main()->chunk_list()->set_max_queue_size(chunks);
+  m_ptr->main()->chunk_list()->sync_chunks(ChunkList::sync_all | ChunkList::sync_force);
 }
 
 uint32_t
@@ -399,7 +394,7 @@ Download::peers_connected() const {
 
 uint32_t
 Download::peers_not_connected() const {
-  return m_ptr->main()->available_list()->size();
+  return m_ptr->main()->peer_list()->available_list()->size();
 }
 
 uint32_t
@@ -424,7 +419,7 @@ Download::peers_currently_interested() const {
 
 bool
 Download::accepting_new_peers() const {
-  return m_ptr->info()->accepting_new_peers();
+  return m_ptr->info()->is_accepting_new_peers();
 }
 
 uint32_t
@@ -486,26 +481,22 @@ Download::update_priorities() {
 
 void
 Download::peer_list(PList& pList) {
-  std::for_each(m_ptr->main()->connection_list()->begin(),
-		m_ptr->main()->connection_list()->end(),
-		rak::bind1st(std::mem_fun<void,PList,PList::const_reference>(&PList::push_back), &pList));
+  std::for_each(m_ptr->main()->connection_list()->begin(), m_ptr->main()->connection_list()->end(),
+                rak::bind1st(std::mem_fun<void,PList,PList::const_reference>(&PList::push_back), &pList));
 }
 
 Peer
 Download::peer_find(const std::string& id) {
   ConnectionList::iterator itr =
-    std::find_if(m_ptr->main()->connection_list()->begin(),
-		 m_ptr->main()->connection_list()->end(),
-		 
-		 rak::equal(id, rak::on(std::mem_fun(&PeerConnectionBase::peer_info),
-					std::mem_fun(&PeerInfo::get_id))));
+    std::find_if(m_ptr->main()->connection_list()->begin(), m_ptr->main()->connection_list()->end(),
+                 rak::equal(id, rak::on(std::mem_fun(&PeerConnectionBase::c_peer_info), std::mem_fun(&PeerInfo::id))));
 
   return itr != m_ptr->main()->connection_list()->end() ? *itr : NULL;
 }
 
 void
 Download::disconnect_peer(Peer p) {
-  m_ptr->main()->connection_list()->erase(p.ptr());
+  m_ptr->main()->connection_list()->erase(p.ptr(), 0);
 }
 
 sigc::connection
